<?php

namespace Drupal\bibcite_import_orcid\Controller;

use Drupal\bibcite_entity\Entity\Reference;
use Drupal\bibcite_entity\Entity\Contributor;
use Drupal\Core\Controller\ControllerBase;
use RenanBr\BibTexParser\Listener;
use RenanBr\BibTexParser\Parser;
use RenanBr\BibTexParser\Exception\ParserException;
use RenanBr\BibTexParser\Exception\ProcessorException;

class OrcidImporterPageController extends ControllerBase{

  const LOG_CHANNEL = 'bibcite_import_orcid';

  private $bibcite_work_data = [];

  /** @const */
  private $bibcite_entity_types = [
      "annotation" => "miscellaneous",
      "artistic-performance" => "miscellaneous",
      "book-chapter" => "book_chapter",
      "book-review" => "miscellaneous",
      "book" => "book",
      "conference-abstract" => "conference_proceedings",
      "conference-paper" => "conference_paper",
      "conference-poster" => "miscellaneous",
      "data-set" => "database",
      "dictionary-entry" => "miscellaneous",
      "disclosure" => "miscellaneous",
      "dissertation-thesis" => "thesis",
      "edited-book" => "book",
      "encyclopedia-entry" => "miscellaneous",
      "invention" => "miscellaneous",
      "journal-article" => "journal_article",
      "journal-issue" => "journal",
      "lecture-speech" => "presentation",
      "license" => "miscellaneous",
      "magazine-article" => "magazine_article",
      "manual" => "miscellaneous",
      "newsletter-article" => "miscellaneous",
      "newspaper-article" => "newspaper_article",
      "online-resource" => "website",
      "other" => "miscellaneous",
      "patent" => "patent",
      "physical-object" => "miscellaneous",
      "presentation" => "presentation",
      "preprint" => "preprint",
      "registered-copyright" => "miscellaneous",
      "report" => "report",
      "research-technique" => "miscellaneous",
      "research-tool" => "miscellaneous",
      "software" => "software",
      "spin-off-company" => "miscellaneous",
      "standards-and-policy" => "miscellaneous",
      "supervised-student-publication" => "miscellaneous",
      "technical-standard" => "miscellaneous",
      "test" => "miscellaneous",
      "trademark" => "miscellaneous",
      "translation" => "miscellaneous",
      "website" => "website",
      "working-paper" => "miscellaneous",
      "undefined" => "miscellaneous",
  ];

  public function content($user_orcid_id) {

    $result = \Drupal::service("bibcite_import_orcid.importer")->importOrcidData($user_orcid_id);

    $markup = $this->generateMarkup($result, $user_orcid_id);

    return $markup;
  }

  function generateMarkup($result, $user_orcid_id){

    $markup =   "<div class='container'>";
    $markup .=  "<h2>Resultado da importação para o ORCID " . $user_orcid_id . ":</h2>";
    $markup .=    "<div>";

    if(intval($result['updated'])>0){
      $markup .=      "<p> Foram actualizadas " . $result['updated'] . " referências.</p>";
    }

    if(intval($result['created'])>0){
      $markup .=    "<p> Foram criadas " . $result['created'] . " referências.</p>";
    }

    if(count($result['entities'])>0){

      $markup .= "<ul>";

      foreach($result['entities'] as $ref_id){
        $ref_entity = Reference::load($ref_id);
        $markup .= "<li><a href='/bibcite/reference/".$ref_id."'>" . $ref_entity->get('title')->getValue()[0]['value'] . "</a></li>";
      }

      $markup .= "</ul>";
    }
    $markup .=    "</div>";
    $markup .=  "</div>";

    return [
      '#markup' => $markup,
    ];
  }

  public function getOrcidWorkData($work_put_code, $orcid)
  {
      $url = "https://pub.orcid.org/" . $orcid . "/works/" . $work_put_code;

      $ch = curl_init();

      curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($ch, CURLOPT_URL, $url);
      curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
      curl_setopt($ch, CURLOPT_TIMEOUT, 600);
      curl_setopt($ch, CURLOPT_HTTPHEADER, [
          'Accept: application/vnd.orcid+json',
      ]);

      $curl_result = curl_exec($ch);

      if (curl_error($ch)) {
          $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
          \Drupal::logger(self::LOG_CHANNEL)->notice("cURL error while fetching ORCID works for user " . $orcid . ": " . curl_error($ch));
          if ($http_code == 504) {
              \Drupal::logger(self::LOG_CHANNEL)->notice("ORCID API Timeout");
          }
      }

      curl_close($ch);

      $json_data = json_decode($curl_result, true);

      return $json_data['bulk'][0]['work'];
  }
  public function getOrcidPersonData($orcid)
  {
    //Set the url for ORCID user data
    $url = "https://pub.orcid.org/" . $orcid;
    //Initialize cURL session
    $ch = curl_init();
    //Define cURL options
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
    curl_setopt($ch, CURLOPT_TIMEOUT, 600);
    curl_setopt($ch, CURLOPT_HTTPHEADER, [
        'Accept: application/vnd.orcid+json',
    ]);
    //Receive the result of the cURL session
    $curl_result = curl_exec($ch);
    //Check if there were any errors during the curl session
    if (curl_error($ch)) {
        $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        \Drupal::logger(self::LOG_CHANNEL)->notice("cURL error while fetching ORCID works for user " . $orcid . ": " . curl_error($ch));
        if ($http_code == 504) {
            \Drupal::logger(self::LOG_CHANNEL)->notice("ORCID API Timeout");
        }
    }
    //Close cURL session
    curl_close($ch);

    //Decodes the information to JSON
    $json_data = json_decode($curl_result, true);
    //Receive the data from the user
    $person_data = $json_data['person'];

    return $person_data;
  }
  // Map ref types
  public function bibcite_entity_types($entity){
    $pubs = [
        "annotation" => "miscellaneous",
        "artistic-performance" => "miscellaneous",
        "book-chapter" => "book_chapter",
        "book-review" => "miscellaneous",
        "book" => "book",
        "conference-abstract" => "conference_proceedings",
        "conference-paper" => "conference_paper",
        "conference-poster" => "miscellaneous",
        "data-set" => "database",
        "dictionary-entry" => "miscellaneous",
        "disclosure" => "miscellaneous",
        "dissertation-thesis" => "thesis",
        "edited-book" => "book",
        "encyclopedia-entry" => "miscellaneous",
        "invention" => "miscellaneous",
        "journal-article" => "journal_article",
        "journal-issue" => "journal",
        "lecture-speech" => "presentation",
        "license" => "miscellaneous",
        "magazine-article" => "magazine_article",
        "manual" => "miscellaneous",
        "newsletter-article" => "miscellaneous",
        "newspaper-article" => "newspaper_article",
        "online-resource" => "website",
        "other" => "miscellaneous",
        "patent" => "patent",
        "physical-object" => "miscellaneous",
        "presentation" => "presentation",
        "preprint" => "preprint",
        "registered-copyright" => "miscellaneous",
        "report" => "report",
        "research-technique" => "miscellaneous",
        "research-tool" => "miscellaneous",
        "software" => "software",
        "spin-off-company" => "miscellaneous",
        "standards-and-policy" => "miscellaneous",
        "supervised-student-publication" => "miscellaneous",
        "technical-standard" => "miscellaneous",
        "test" => "miscellaneous",
        "trademark" => "miscellaneous",
        "translation" => "miscellaneous",
        "website" => "website",
        "working-paper" => "miscellaneous",
        "undefined" => "miscellaneous",
    ];
    if (array_key_exists($entity, $pubs)) {
      return $pubs[$entity];
    } else {
        return "miscellaneous";
    }
  }
  public function parce_bibtex($work){
    // handle bibtex if it exists
    try {
      $parser = new Parser();
      $listener = new Listener();

      //remove accents so it can pass on the BibTexParser
      $unwanted_array = array(
          'Š' => 'S', 'š' => 's', 'Ž' => 'Z', 'ž' => 'z', 'À' => 'A', 'Á' => 'A', 'Â' => 'A', 'Ã' => 'A', 'Ä' => 'A', 'Å' => 'A', 'Æ' => 'A', 'Ç' => 'C', 'È' => 'E', 'É' => 'E',
          'Ê' => 'E', 'Ë' => 'E', 'Ì' => 'I', 'Í' => 'I', 'Î' => 'I', 'Ï' => 'I', 'Ñ' => 'N', 'Ò' => 'O', 'Ó' => 'O', 'Ô' => 'O', 'Õ' => 'O', 'Ö' => 'O', 'Ø' => 'O', 'Ù' => 'U',
          'Ú' => 'U', 'Û' => 'U', 'Ü' => 'U', 'Ý' => 'Y', 'Þ' => 'B', 'ß' => 'Ss', 'à' => 'a', 'á' => 'a', 'â' => 'a', 'ã' => 'a', 'ä' => 'a', 'å' => 'a', 'æ' => 'a', 'ç' => 'c',
          'è' => 'e', 'é' => 'e', 'ê' => 'e', 'ë' => 'e', 'ì' => 'i', 'í' => 'i', 'î' => 'i', 'ï' => 'i', 'ð' => 'o', 'ñ' => 'n', 'ò' => 'o', 'ó' => 'o', 'ô' => 'o', 'õ' => 'o',
          'ö' => 'o', 'ø' => 'o', 'ù' => 'u', 'ú' => 'u', 'û' => 'u', 'ý' => 'y', 'þ' => 'b', 'ÿ' => 'y'
      );

      $work = preg_replace('/\s+/', '', strtr($work, $unwanted_array));

      $parser->addListener($listener);
      $parser->parseString($work);
      $reference_data = [];
      $entries = $listener->export();

      foreach ($entries[0] as $prop => $value) {
          switch ($prop) {
              case 'pages':
              case 'Pages':
              case 'PAGES':
                $reference_data['bibcite_pages'] = $entries[0][$prop];
                break;
              case 'number':
              case 'Number':
              case 'NUMBER':
                $reference_data['bibcite_issue'] = $entries[0][$prop];
                break;
              case 'volume':
              case 'Volume':
              case 'VOLUME':
                $reference_data['bibcite_volume'] = $entries[0][$prop];
                break;
              case 'author':
              case 'Author':
              case 'AUTHOR':
                $reference_data['bibcite_authors'] = explode('and',$entries[0][$prop]);
                break;
              case 'booktitle':
              case 'Booktitle':
              case 'BookTitle':
              case 'BOOKTITLE':
                $reference_data['bibcite_booktitle'] = $entries[0][$prop];
                break;
              case 'series':
              case 'Series':
              case 'SERIES':
                $reference_data['bibcite_section'] = $entries[0][$prop];
                break;
              case 'publisher':
              case 'Publisher':
              case 'PUBLISHER':
                $reference_data['bibcite_publisher'] = $entries[0][$prop];
                break;
          }
      }
      return $reference_data;
    } catch (ParserException $exception) {
        \Drupal::logger('bibcite_import_orcid')->error("Parser Exception: " . $exception->getMessage());
    } catch (ProcessorException $exception) {
        \Drupal::logger('bibcite_import_orcid')->error("Processor Exception: " . $exception->getMessage());
    }
  }
  // Get a name and return id (existing or created)
  public function author_to_id($name){
    if(!is_int($name)){
      if (strpos($name, ",")) {
        $names = array_reverse(explode(", ", $name));
      } else {
          $names = explode(" ", $name);
      }
      $cfname = reset($names);
      $clname = ucfirst(end($names));
      $id = \Drupal::entityQuery('bibcite_contributor')
      ->condition('first_name', $cfname)
      ->condition('last_name', $clname)
      ->execute();

      if(empty($id)){
          //Create a contributor
          $c = Contributor::create([
              'type' => 'entity_contributor',
              'last_name' => $clname,
              'first_name' => $cfname,
          ]);
          $c->save();
          $id = $c->id();
      }

    } else {
      $id = $name;
    }
    return reset($id);
  }
}
