<?php

namespace Drupal\bibcite_import_orcid\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Routing\TrustedRedirectResponse;


/**
 * Provides a ORCID Import form.
 */
class ImportForm extends FormBase
{
    use StringTranslationTrait;

    /**
     * {@inheritdoc}
     */
    public function getFormId()
    {
        return 'bibcite_import_orcid_import';
    }

    /**
     * {@inheritdoc}
     */
    public function buildForm(array $form, FormStateInterface $form_state, $id = NULL)
    {
        $uid = \Drupal::currentUser()->id();
        $works = $_SESSION['works_data_' . $id];
        if(!empty($works)){
            $form['works'] = [
                "#tree" => true,
                "#prefix" => '<div class="container"><div class="ign_actions"><a class="ignore_all">Ignore all</a><a class="ignore_none">Ignore none</a></div>',
                "#suffix" => '</div>',
            ];

            foreach ($works as $item) {
                $work = json_decode($item);
                $title = $work->title;
                $options = (array)$work->contribs;
                $user_ids = $work->user_authors;

                $element = [];

                $element['work'] = array(
                    '#type' => 'fieldset',
                    '#title' => $title,
                );

                $element['work']['ignore'] = array(
                    '#type' => 'checkbox',
                    '#title' => t('Ignore this publication'),
                );
                $element['reference_works'][] = [
                    '#type' => 'textarea',
                    '#value' => $item,
                    '#attributes' => [
                        'class' => ['hidden'],
                    ],
                ];
                $pub_user = array_flip((array)$options);
                $com = array_intersect((array)$user_ids,$pub_user);
                if(count($com) > 0 ){
                    $element['work']['own'] = [
                        '#type' => 'markup',
                        '#markup' => '<div class="own">Already linked to the current user. Only info will be updated.</div>'
                    ];
                }else{
                    $element['work']['authors'] = [
                        '#type' => 'radios',
                        '#title' => t('Authors'),
                        '#options' => $options,
                    ];
                    $attribs = $work->contribs_atts;
                    if ($attribs) {
    
                        $element['work']['authors']['#options_attributes'][key($attribs)] = [
                            "disabled" => "disabled",
                        ];
                    }
                }  

                $form['works'][] = $element;
            
            }

            //If there are no publications to show.
            if (empty($works)) {
                $form['#attached']['library'][] = "bibcite_import_orcid/orcid-import-block";
                $form['works'][] = [
                    '#type' => 'text',
                    "#prefix" => '<div class="orcid-no-results">No publications to import.<br><a href="/user/' . $uid . '">Return to the profile page</a></div>',
                ];
            } else {
                $form['actions'] = [
                    '#type' => 'actions',
                ];
                $form['actions']['submit'] = [
                    '#type' => 'submit',
                    '#value' => $this->t('Import'),
                    "#prefix" => '<div class="container">',
                    "#suffix" => '</div>',
                ];
            }
            $form['#attached']['library'][] = 'bibcite_import_orcid/orcid-import-block';
            return $form;
        } else {
            return new TrustedRedirectResponse('/user');
        }

    }

    /**
     * {@inheritdoc}
     */
    public function validateForm(array &$form, FormStateInterface $form_state)
    {
        $works = $form_state->getValue('works');
        foreach ($works as $key => $value) {
            if($value["work"]["ignore"]===0 && isset($value["work"]["authors"]) && $value["work"]["authors"]===null){
                $form_state->setErrorByName('authors', $this->t('A contributor must be selected for each publication not set to be ignored.'));
            }
        }
    }

    /**
     * {@inheritdoc}
     */
    public function submitForm(array &$form, FormStateInterface $form_state)
    {
        //Get the uid of the current user
        $uid = \Drupal::currentUser()->id();
        //receive all the values from the form_state
        $all_values = $form_state->getValues();
        $form_works = $all_values['works'];

        // array for the batch
        $operations = [];

        foreach ($form_works as $key => $value) {
            // Cleanup, add to queue
            if ($value['work']["ignore"]===0) {
                $value['pub'] = $_SESSION["works_data"][$key];
                $value['author'] = $value['work']['authors'];
                $operations[] = ['bibcite_import_orcid_batch', [$value, $uid]];
            }
        }

        $batch = array(
            'title' => t('Preparing import form, please wait...'),
            'operations' => $operations,
            'finished' => 'bibcite_import_orcid_batch_finished',
            'init_message' => t('Importing'),
            'progress_message' => t('@current of @total. Estimated duration: @estimate.'),
            'error_message' => t('There was an error on the import process.'),
            'file' => drupal_get_path('module', 'bibcite_import_orcid') . '/bibcite_import_orcid.batch.inc',
        );

        batch_set($batch);
    }

}
